//-----------------------------LICENSE NOTICE------------------------------------
//
// This file is part of Firewall Wars.
//
//     FirewallWars is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License.
// 
//     FirewallWars is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with FirewallWars.  If not, see <http://www.gnu.org/licenses/>.
// 
//------------------------------------------------------------------------------
#include "paint.h"

//tabla de trasparencias
cpctm_createTransparentMaskTable(g_tablatrans, 0x500,M0,2);

//dibujar
//dibujar en pantalla
void dibujarMapa(u8* mapa) __z88dk_fastcall {
  cpct_etm_drawTilemap2x4(g_map1_W,g_map1_H,ORIGEN_MAPA,mapa);
}

//Dibuja al prota con transparencias
void dibujarProta(TJugador* prota) __z88dk_fastcall {
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START,prota->x,prota->y);
	cpct_drawSpriteMaskedAlignedTable (prota->sprite,pvmem,G_SPRITESJUGADORES_1_W,G_SPRITESJUGADORES_1_H,g_tablatrans);
}

//Dibuja al enemigo con transparencias
void dibujarEnemigo(Tenemy* ene) __z88dk_fastcall {
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START,ene->x,ene->y);
	cpct_drawSpriteMaskedAlignedTable (ene->sprite,pvmem,G_SPRITESJUGADORES_2_W,G_SPRITESJUGADORES_2_H,g_tablatrans);
}

//Dibuja el objeto con transparencias
void dibujarObjeto(TCinta* ci) __z88dk_fastcall {
	u8* pvmem = cpct_getScreenPtr(CPCT_VMEM_START,ci->x,ci->y);
	if(ci->cola){
	    cpct_drawSpriteMaskedAlignedTable (ci->sprite,pvmem,G_COLA_W,G_COLA_H,g_tablatrans);
	}else{
	    cpct_drawSpriteMaskedAlignedTable (ci->sprite,pvmem,G_OBJETO_W,G_OBJETO_H,g_tablatrans);
	}
}

void dibujarTodosObjetos(TCinta* cintas[NUM_CINTAS], u8 nmapa,u8 arriba,u8 floor){
	u8 i = 0;
	for(;i < NUM_CINTAS;++i){
		if(cintas[i]->locationMap == nmapa){
			if(cintas[i]->arriba == arriba){
				if(cintas[i]->floor == floor){
					if(cintas[i]->inMap)
					dibujarObjeto(cintas[i]);
				}
			}
		}
	}
}

void dibujarTodosEnemigos(Tenemy* enemigos[NUM_ENEMIGOS_NIVEL], u8 nmapa,u8 arriba,u8 floor){
	u8 i = 0;
	for(;i < NUM_ENEMIGOS_NIVEL;++i){
		if(enemigos[i]->locationMap == nmapa){
			if(enemigos[i]->arriba == arriba){
				if(enemigos[i]->floor == floor){
					dibujarEnemigo(enemigos[i]);
				}				
			}
			
		}
	}
}

void dibujarCPC(){
	u8* memCPC = cpct_getScreenPtr(CPCT_VMEM_START,POSXCPC,POSYCPC);
	cpct_drawSprite(g_CPC,memCPC,G_CPC_W,G_CPC_H);
}

void dibujarEscaleras(){
	u8* memES = cpct_getScreenPtr(CPCT_VMEM_START,POSXES,POSYES);
	cpct_drawSprite(g_escaleras,memES,G_ESCALERAS_W,G_ESCALERAS_H);
}

//Borra el objeto de su posición anterior
void borrarObjeto(TCinta* ci, u8* mapa){
	u8 w,h;
	if(ci->cola){
		//par impar
		w = 6 + (ci->x & 1);
		//modulo 4 si esta entre tiles suma uno si no lo dibuja en el que es 
		h = 4 + (ci->y & 3 ? 1 : 0);
	}
	else{
		//par impar
		w = 2 + (ci->x & 1);
		//modulo 4 si esta entre tiles suma uno si no lo dibuja en el que es 
		h = 4 + (ci->y & 3 ? 1 : 0);
	}
	cpct_etm_drawTileBox2x4(ci->x /2,(ci->y - ORIGEN_MAPA_Y)/4,w,h,g_map1_W,ORIGEN_MAPA,mapa);
}

void redibujarEnemigo(Tenemy* ene, u8* mapa,u8 nmapa,u8 arriba){
	if(ene->locationMap==nmapa){
		if(ene->arriba == arriba){
		borrarEnemigo(ene,mapa);
		ene->px = ene->x;
		ene->py = ene->y;
		dibujarEnemigo(ene);
		}
	}
}

//Borra al enemigo de su posición anterior
void borrarEnemigo(Tenemy* ene, u8* mapa){
	u8 w = 1 + (ene->px & 1);
	u8 h = 2 + (ene->py & 3 ? 1 : 0);
	cpct_etm_drawTileBox2x4(ene->px /2,(ene->py - ORIGEN_MAPA_Y)/4,w,h,g_map1_W,ORIGEN_MAPA,mapa);
}

//Redibuja al protagonista borrandolo de su posicion anterior y dibujandolo en la nueva
void redibujarProta(TJugador* prota, u8* mapa,TCinta* cintas[NUM_CINTAS]){
	borrarProta(prota,mapa);
	prota->px = prota->x;
	prota->py = prota->y;
  	enContactoCinta(cintas,prota,mapa);
	dibujarProta(prota);
}

//Borra al protagonista de su posición anterior
void borrarProta(TJugador* prota, u8* mapa){
	u8 w = 1 + (prota->px & 1);
	u8 h = 2 + (prota->py & 3 ? 1 : 0);
	cpct_etm_drawTileBox2x4(prota->px /2,(prota->py - ORIGEN_MAPA_Y)/4,w,h,g_map1_W,ORIGEN_MAPA,mapa);
}

//mecanicas
void invisible(TJugador* prota) __z88dk_fastcall {
	if(prota->state){
		prota->state = 0;
		prota->sprite = g_spritesJugadores_1;
		prota->mover = SI;

	}else{
		if(prota->energy > 0){
			prota->state = 1;
			prota->sprite = g_spritesJugadores_0;
			prota->mover = SI;
		}
		
	}
}

void waitXHalts(u16 wait) __z88dk_fastcall{
	while(--wait){
		__asm__("halt");
	}
}

//Se llama a esta función cuando un protagonista es tocado por el enemigo, justo antes de terminar el juego
void animacionPerdido(TJugador* prota,Tenemy* ene,u8* mapa){
	cpct_akp_musicInit(dead_mysong);
	cpct_akp_musicPlay();
	borrarEnemigo(ene,mapa);
	waitXHalts(60);
	prota->sprite=g_spritesJugadores_2;
	borrarProta(prota,mapa);
	dibujarProta(prota);
	cpct_waitVSYNC();
	waitXHalts(60);
	prota->sprite=g_spritesJugadores_3;
	borrarProta(prota,mapa);
	dibujarProta(prota);
	cpct_waitVSYNC();
	waitXHalts(60);
	prota->sprite=g_spritesJugadores_4;
	borrarProta(prota,mapa);
	dibujarProta(prota);
	cpct_waitVSYNC();
	waitXHalts(60);
	borrarProta(prota,mapa);
	cpct_waitVSYNC();
	waitXHalts(60);
	cpct_akp_stop();

}

//se le pasa la x y la y de donde empieza el muro
//la direccion 1 arriba,2 derecha,3 abajo,4 derecha
// y el numero de bloques de 4x4 que dibuja



//Dibuja nuestro rótulo con el cuadrado en el que irán los menús
void dibujarRotulo(u8* memptr)__z88dk_fastcall{
	cpct_drawSprite (g_menu,memptr,G_MENU_W, G_MENU_H);

	//Cuadrado negro
	memptr= cpct_getScreenPtr(CPCT_VMEM_START,1,80);
	cpct_drawSolidBox(memptr,0,62,116);
	memptr= cpct_getScreenPtr(CPCT_VMEM_START,16,80);
	cpct_drawSolidBox(memptr,0,62,116);
}

void imprimeMenu(){
	u8* memptr;
	memptr= cpct_getScreenPtr(CPCT_VMEM_START,20,10);
	dibujarRotulo(memptr);

	memptr=cpct_getScreenPtr(CPCT_VMEM_START,13,100);
	cpct_drawStringM0("1. Controls",memptr,15,0);

	memptr=cpct_getScreenPtr(CPCT_VMEM_START,13,120);
	cpct_drawStringM0("2. Change Keys",memptr,15,0);

	memptr=cpct_getScreenPtr(CPCT_VMEM_START,13,140);
	cpct_drawStringM0("3. Credits",memptr,15,0);

	memptr=cpct_getScreenPtr(CPCT_VMEM_START,12,170);
	cpct_drawStringM0("INTRO TO START",memptr,15,0);
}
const u8 paleta[4]={HW_BLACK,HW_BLUE,HW_BRIGHT_BLUE,HW_BRIGHT_YELLOW};


void imprimeAMSTRAD(){
		u8* memptr= cpct_getScreenPtr(CPCT_VMEM_START,0,0);
		cpct_setVideoMode(1);//Cambiamos el modo de impresión
		cpct_setPalette(paleta,4);//Seteamos la paleta de modo 1 que hay que declarar con los coloes {0 2 24 26}
		cpct_setBorder(HW_BLUE);
		cpct_clearScreen(cpct_px2byteM1 (1,1,1,1));//Limpiamos la pantalla
		cpct_drawStringM1 ("Out Bus of TRaMP Corp. Database (v72)", memptr, 3, 1);//Fondo Azul y letras amarillas
		memptr=cpct_getScreenPtr(CPCT_VMEM_START,0,10);
		cpct_drawStringM1 ("2017 TRaMP Corp. & Associated", memptr, 3, 1);//Fondo Azul y letras amarillas
		memptr=cpct_getScreenPtr(CPCT_VMEM_START,0,20);
		cpct_drawStringM1 ("Coffee ParaDOS v3.14.15C 2009 TRaMP Corp.", memptr, 3, 1);//Fondo Azul y letras amarillas
		memptr=cpct_getScreenPtr(CPCT_VMEM_START,0,30);
		cpct_drawStringM1 ("Zilog Z80 & CPCTELERA v1.5", memptr, 3, 1);//Fondo Azul y letras amarillas
		memptr=cpct_getScreenPtr(CPCT_VMEM_START,0,40);
		cpct_drawStringM1 ("Ready", memptr, 3, 1);//Fondo Azul y letras blancas
		//Aqui esperar a que introduzcan el comando correcto
		do{
		    cpct_scanKeyboard_f();
		}while(! (cpct_isKeyPressed(Key_Control) && (cpct_isKeyPressed(Key_Enter) || cpct_isKeyPressed(Key_Return))));
		memptr=cpct_getScreenPtr(CPCT_VMEM_START,0,50);
		printString2("run\"ABORTCODE.BIN",memptr);
		waitXHalts(600);
		memptr=cpct_getScreenPtr(CPCT_VMEM_START,0,60);
		printString2("Running code",memptr);
		waitXHalts(600);
		memptr=cpct_getScreenPtr(CPCT_VMEM_START,0,70);
		printString2("Missile launch successfully aborted.",memptr);
		waitXHalts(600);
		cpct_clearScreen(cpct_px2byteM1 (1,1,1,1));//Limpiamos la pantalla
		cpct_setPalette(g_palette,16);
		cpct_setVideoMode(0);//Cambiamos el modo de impresión
}

void printString2(const u8* text1,u8* memptr){
	imprimeStringSin(memptr,3,1,text1);//Con el 2 o 3 se ve bastante bien
}


//varieble para sincronizar la musica
u8 syncHander;

void initHandler25(){
	if(++syncHander == 12){
		play();
		syncHander = 0;
	}
}

void endHandler(){
	cpct_akp_stop ();
}